import { BN } from '@coral-xyz/anchor';
import { createTransferCheckedInstruction, } from '@solana/spl-token';
import { isAta } from '@render-foundation/spl-utils';
import { RENDER_MINT } from './constants';
import { treeConfigKey, makerKey, collectionKey, accountEscrowKey, userAccountKey, } from './pdas';
import AWS from 'aws-sdk';
export const initializeMakerV0Ix = (program, args, accts) => {
    const maker = makerKey(args.name)[0];
    const collection = collectionKey(maker)[0];
    return (program.methods
        .initializeMakerV0({
        updateAuthority: args.updateAuthority,
        issuingAuthority: args.issuingAuthority ?? args.updateAuthority,
        name: args.name,
        symbol: args.symbol,
        metadataUri: args.metadataUri,
    })
        .accounts({
        payer: accts.payer ?? args.updateAuthority,
        maker,
        collection,
    })
        .instruction());
};
export const addTreeV0Ix = (program, args, accts) => {
    const maker = makerKey(args.makerName)[0];
    const treeConfig = treeConfigKey(accts.merkleTree)[0];
    return program.methods
        .addTreeV0({
        maxDepth: args.maxDepth,
        maxBufferSize: args.maxBufferSize,
    })
        .accounts({
        updateAuthority: accts.updateAuthority,
        payer: accts.payer ?? accts.updateAuthority,
        maker,
        merkleTree: accts.merkleTree,
        treeConfig,
    })
        .instruction();
};
export const initializeUserV0Ix = (program, { accountId, }, { renderMint, authority, payer, }) => {
    return program.methods
        .initializeUserV0({ accountId })
        .accounts({
        authority,
        payer: payer ?? authority,
        userAccount: userAccountKey(accountId)[0],
        accountEscrow: accountEscrowKey(accountId, renderMint)[0],
        renderMint: renderMint ?? RENDER_MINT,
    })
        .instruction();
};
export const transferToAccountEscrowIx = async (conn, { userId, amount, fromAta, from, mint = RENDER_MINT, decimals = 8, }) => {
    const ata = await isAta(conn, fromAta);
    if (!ata) {
        throw new Error(`from ${fromAta} is not an ata`);
    }
    return createTransferCheckedInstruction(fromAta, mint, accountEscrowKey(userId, mint)[0], ata.owner, amount, decimals);
};
export const createJobV0Ix = async (program, { accountId, makerName, job, }, accts) => {
    let uri = job.uri;
    if (typeof uri === 'object') {
        // @ts-ignore
        const { accessKey, secretKey, data } = job.uri;
        const bucket = 'render-artist-jobs';
        const s3 = new AWS.S3({
            accessKeyId: accessKey,
            secretAccessKey: secretKey,
        });
        const params = {
            Bucket: bucket,
            Key: `${job.id}.json`,
            Body: JSON.stringify(data),
            ContentType: 'application/json',
        };
        await s3.putObject(params).promise();
        uri = `https://${bucket}.s3.us-east-1.amazonaws.com/${job.id}.json`;
        console.log(`upload ${job.id} to ${uri}`);
    }
    const maker = makerKey(makerName)[0];
    const collection = collectionKey(maker)[0];
    const treeConfig = treeConfigKey(accts.merkleTree)[0];
    return program.methods
        .createJobV0({
        jobId: job.id,
        uri: uri,
        accountId,
    })
        .accounts({
        issuingAuthority: accts.issuingAuthority,
        payer: accts.payer ?? accts.issuingAuthority,
        maker,
        merkleTree: accts.merkleTree,
        collection,
        treeConfig,
        //bubblegumSigner: accts.issuingAuthority
    })
        .instruction();
};
export const burnJobRenderV0Ix = (program, { accountId, jobs, }, accts) => {
    return program.methods
        .burnJobRenderV0({
        accountId,
        jobs: jobs.map((job) => ({
            id: job.id,
            renderSpent: new BN(job.renderSpent),
            completedAt: new BN(Math.trunc(job.completedAt.getTime() / 1000)),
        })),
    })
        .accounts({
        authority: accts.authority,
        renderMint: accts.renderMint,
    })
        .instruction();
};
//# sourceMappingURL=instructions.js.map